/*----------------    FILE HEADER  ------------------------------------------

This file is part of monogis.
Copyright (C) 2004-2006 by: http://sourceforge.net/projects/mgis


This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA             
---------------------------------------------------------------------------*/

/*-------------- Original license ------------------------------------------
The monoGIS GDAL driver is a collection of .NET classes written in C# that
implement the fundamental P/Invoke operations required to treat 
geo-spatial data using Frank Wamerdam's GDAL C++ library.

This collection of classes is a porting (with some additions and modifications) of
code that has been sent to the gdal-dev mailing list (gdal-dev@lists.maptools.org). 
We want especially thank to:
- Simon Perkins, for contributing his GDAL_csharp library to gdal-dev:
------ GDAL_csharp "license" ---
"This partial implementation is good enough for my needs now. If anyone 
else wants to use it, please feel free. If you want to extend it and 
send updates to me, great! The code doesn't contain any explcit licence 
headers, but Michael was suggesting an MIT licence, same as GDAL. Fine 
with me.

Cheers,

Sy"

 * ------ GDAL_csharp "license" ---
- Morton Nielsen, for contributing code to the gdal-dev:
"
Great work on the .NET wrappers. I decided to take it a step further and putting it all
in some nice classes, so you won't have to worry about pointers, disposing objects etc.
I think a lot of .NET developers would like that approach. 
I put the code example for download here:
     http://www.iter.dk/dotnet/gdal/gdal_test.zip

I added some code-comments copied from the gdal website as well, 
so you will have intellisence on most of the methods.
"
------ gdal_test "license" ----------------------------------------------------
 * ---------------------------------

Copyright (c) 2005 Morten Nielsen

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is furnished
to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies of this Software or works derived from this Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
SOFTWARE.

------ gdal_test "license" ------------------------------------------------------


 
Copy of GDAL/OGR license:
  Copyright (c) 2000, Frank Warmerdam

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE. 
---------------------------------------------------------------------------*/
using System;
using System.Text;
using System.IO;
using System.Runtime.InteropServices;
using System.Collections.Specialized;

namespace Gdal
{
    public enum Access
    {
        ReadOnly = 0,
        Update = 1,
    }

    public class Dataset : IDisposable
    {
        internal HandleRef hDataset;

        /// <summary>
        /// Construct a Dataset that is associated with an existing file.
        /// </summary>
        /// <param name="filename">Filename to open</param>
        /// <param name="access">Access mode with which to open file</param>
        public Dataset(string filename, Access access)
        {
            hDataset = new HandleRef(this, InterOp.GDALOpen(filename, access));
            if (!IsValid)
            {
               throw new IOException("Unable to open file " + filename + "\n" +
                    InterOp.CPLGetLastErrorMsg());
            }
        }

        /// <summary>
        /// Construct a Dataset that is associated with an existing file, with ReadOnly access.
        /// </summary>
        /// <param name="filename"></param>
        public Dataset(string filename) : this(filename, Access.ReadOnly) { }

        internal Dataset(IntPtr ptrDataset)
        {
            hDataset = new HandleRef(this, ptrDataset);
        }

        /// <summary>
        /// True if the Dataset was successfully opened / created and has not been closed.
        /// </summary>
        public bool IsValid { get { return hDataset.Handle != IntPtr.Zero; } }

        /// <summary>
        /// The filename associated with this Dataset.
        /// </summary>
        public string Filename { get { return InterOp.GDALGetDescription(hDataset); } }

        /// <summary>
        /// The access with which this Dataset was opened.
        /// </summary>
        public Access Access { get { return (Access)InterOp.GDALGetAccess(hDataset); } }

        /// <summary>
        /// Close the Dataset. This method is also called by Dispose(). Do not invoke any methods
        /// on a Dataset after closing it!
        /// </summary>
        public void Close()
        {
            if (hDataset.Handle != IntPtr.Zero) {
                InterOp.GDALClose(hDataset);
                hDataset = new HandleRef(this, IntPtr.Zero);
            }
        }

        /// <summary>
        /// The width of the image.
        /// </summary>
        public int XSize { get { return InterOp.GDALGetRasterXSize(hDataset); } }

        /// <summary>
        /// The height of the image.
        /// </summary>
        public int YSize { get { return InterOp.GDALGetRasterYSize(hDataset); } }

        /// <summary>
        /// The width and height as a System.Drawing.Size object.
        /// </summary>
        public System.Drawing.Size ImageSize { get { return new System.Drawing.Size(XSize, YSize); } }

        /// <summary>
        /// The number of bands in the image.
        /// </summary>
        public int RasterCount { get { return InterOp.GDALGetRasterCount(hDataset); } }

        /// <summary>
        /// Return a RasterBand object for the i'th band. The first band has index 1.
        /// </summary>
        /// <param name="i"></param>
        /// <returns></returns>
        public RasterBand GetRasterBand(int i)
        {
            if (i > RasterCount || i < 1)
                throw new ApplicationException("Gdal.Dataset.GetRasterBand() invoked with invalid index");
            return new RasterBand(InterOp.GDALGetRasterBand(hDataset, i));
        }

        /// <summary>
        /// Fetch the affine transformation coefficients
        /// </summary>
        /// <remarks>
        /// Fetches the coefficients for transforming between pixel/line (X,Y) raster space, and projection coordinates (Xp,Yp) space.<br/>
        /// Xp = T[0] + T[1]*X + T[2]*Y<br/>
        /// Yp = T[3] + T[4]*X + T[5]*Y<br/>
        /// In a north up image, T[1] is the pixel width, and T[5] is the pixel height.
        /// The upper left corner of the upper left pixel is at position (T[0],T[3]).
        /// </remarks>
        /// <returns>Affine transformation parameters</returns>
        public GeoTransform GetGeoTransform()
        {
            double[] rawTransform = new double[6];
            if (InterOp.GDALGetGeoTransform(hDataset, rawTransform) == CPLErr.None)
                return new GeoTransform(rawTransform);
            else
                return new GeoTransform();
        }

        /// <summary>
        /// Set the affine transformation coefficients for this dataset.
        /// </summary>
        /// <param name="transform"></param>
        public void SetGeoTransform(GeoTransform transform)
        {
            // Silently ignore errors for now
            InterOp.GDALSetGeoTransform(hDataset, transform.transform);
        }

        /// <summary>
        /// Get the string describing the projection used by this dataset.
        /// </summary>
        /// <returns></returns>
        public string GetProjection()
        {
            return InterOp.GDALGetProjectionRef(hDataset);
        }

        /// <summary>
        /// Get the Driver object that handles files of the same format as this Dataset.
        /// </summary>
        /// <returns></returns>
        public Driver GetDriver()
        {
            return new Driver(InterOp.GDALGetDatasetDriver(hDataset));
        }

        public /*Dictionary<string, string>*/StringDictionary GetMetadata()
        {
            return InterOp.GDALGetMetadata(hDataset, null);
        }

        public string GetMetadataItem(string key)
        {
            return InterOp.GDALGetMetadataItem(hDataset, key, null);
        }

        /// <summary>
        /// Close the Dataset. Do not invoke any methods on this Dataset after calling Dispose()!
        /// </summary>
        /// <param name="disposing"></param>
        protected void Dispose(bool disposing)
        {
            if (disposing) {
                // dispose managed resources
            }
            Close();
        }

        /// <summary>
        /// Close the Dataset. Do not invoke any methods on this Dataset after calling Dispose()!
        /// </summary>
        /// <param name="disposing"></param>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        ~Dataset()
        {
            Dispose(false);
        }

    }
}
