/*----------------    FILE HEADER  ------------------------------------------

This file is part of monogis.
Copyright (C) 2004-2006 by: http://sourceforge.net/projects/mgis


This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA             
---------------------------------------------------------------------------*/

/*-------------- Original license ------------------------------------------
The monoGIS GDAL driver is a collection of .NET classes written in C# that
implement the fundamental P/Invoke operations required to treat 
geo-spatial data using Frank Wamerdam's GDAL C++ library.

This collection of classes is a porting (with some additions and modifications) of
code that has been sent to the gdal-dev mailing list (gdal-dev@lists.maptools.org). 
We want especially thank to:
- Simon Perkins, for contributing his GDAL_csharp library to gdal-dev:
------ GDAL_csharp "license" ---
"This partial implementation is good enough for my needs now. If anyone 
else wants to use it, please feel free. If you want to extend it and 
send updates to me, great! The code doesn't contain any explcit licence 
headers, but Michael was suggesting an MIT licence, same as GDAL. Fine 
with me.

Cheers,

Sy"

 * ------ GDAL_csharp "license" ---
- Morton Nielsen, for contributing code to the gdal-dev:
"
Great work on the .NET wrappers. I decided to take it a step further and putting it all
in some nice classes, so you won't have to worry about pointers, disposing objects etc.
I think a lot of .NET developers would like that approach. 
I put the code example for download here:
     http://www.iter.dk/dotnet/gdal/gdal_test.zip

I added some code-comments copied from the gdal website as well, 
so you will have intellisence on most of the methods.
"
------ gdal_test "license" ----------------------------------------------------
 * ---------------------------------

Copyright (c) 2005 Morten Nielsen

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is furnished
to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies of this Software or works derived from this Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
SOFTWARE.

------ gdal_test "license" ------------------------------------------------------


 
Copy of GDAL/OGR license:
  Copyright (c) 2000, Frank Warmerdam

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE. 
---------------------------------------------------------------------------*/
using System;
using System.Text;

namespace Gdal
{
    /// <summary>
    /// The coefficients for transforming between pixel/line (X,Y) raster space, and projection coordinates (Xp,Yp) space.<br/>
    /// Xp = T[0] + T[1]*X + T[2]*Y<br/>
    /// Yp = T[3] + T[4]*X + T[5]*Y<br/>
    /// In a north up image, T[1] is the pixel width, and T[5] is the pixel height.
    /// The upper left corner of the upper left pixel is at position (T[0],T[3]).
    /// </summary>
    public class GeoTransform
    {
        internal double[] transform = new double[6];

        public GeoTransform()
        {
            transform = new double[6];
            transform[0] = 0;
            transform[1] = 1;
            transform[2] = 0;
            transform[3] = 0;
            transform[4] = 0;
            transform[5] = 1;
        }

        public GeoTransform(double[] array)
        {
            if (array.Length != 6)
                throw new ApplicationException("GeoTransform constructor invoked with invalid sized array");
            transform = array;
        }

        public double ProjectedX(double imgX, double imgY)
        {
            return transform[0] + transform[1] * imgX + transform[2] * imgY;
        }

        public double ProjectedY(double imgX, double imgY)
        {
            return transform[3] + transform[4] * imgX + transform[5] * imgY;
        }

        public double PixelX(double lat)
        {
            return Math.Abs(transform[0] - lat) / transform[1];
        }

        public double PixelY(double lon)
        {
            return Math.Abs(Math.Abs(transform[3] - lon) / transform[5]);
        }

        public double PixelXwidth(double lat)
        {
            return Math.Abs(lat / transform[1]);
        }

        public double PixelYwidth(double lon)
        {
            return Math.Abs(lon / transform[5]);
        }


        public double this[int i]
        {
            get
            {
                return transform[i];
            }
            set
            {
                transform[i] = value;
            }
        }

        bool IsTrivial
        {
            get
            {
                return transform[0] == 0 && transform[1] == 1 &&
                    transform[2] == 0 && transform[3] == 0 &&
                    transform[4] == 0 && transform[5] == 1;
            }
        }

        public double Left
        {
            get { return transform[0]; }
        }

        public double Top
        {
            get { return transform[3]; }
        }

        public double HorizontalPixelResolution
        {
            get { return transform[1]; }
        }

        public double VerticalPixelResolution
        {
            get { return transform[5]; }
        }

    }
}
