/*----------------    FILE HEADER  ------------------------------------------

This file is part of monogis.
Copyright (C) 2004-2006 by: http://sourceforge.net/projects/mgis


This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA             
---------------------------------------------------------------------------*/

/*-------------- Original license ------------------------------------------
The monoGIS GDAL driver is a collection of .NET classes written in C# that
implement the fundamental P/Invoke operations required to treat 
geo-spatial data using Frank Wamerdam's GDAL C++ library.

This collection of classes is a porting (with some additions and modifications) of
code that has been sent to the gdal-dev mailing list (gdal-dev@lists.maptools.org). 
We want especially thank to:
- Simon Perkins, for contributing his GDAL_csharp library to gdal-dev:
------ GDAL_csharp "license" ---
"This partial implementation is good enough for my needs now. If anyone 
else wants to use it, please feel free. If you want to extend it and 
send updates to me, great! The code doesn't contain any explcit licence 
headers, but Michael was suggesting an MIT licence, same as GDAL. Fine 
with me.

Cheers,

Sy"

 * ------ GDAL_csharp "license" ---
- Morton Nielsen, for contributing code to the gdal-dev:
"
Great work on the .NET wrappers. I decided to take it a step further and putting it all
in some nice classes, so you won't have to worry about pointers, disposing objects etc.
I think a lot of .NET developers would like that approach. 
I put the code example for download here:
     http://www.iter.dk/dotnet/gdal/gdal_test.zip

I added some code-comments copied from the gdal website as well, 
so you will have intellisence on most of the methods.
"
------ gdal_test "license" ----------------------------------------------------
 * ---------------------------------

Copyright (c) 2005 Morten Nielsen

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is furnished
to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies of this Software or works derived from this Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
SOFTWARE.

------ gdal_test "license" ------------------------------------------------------


 
Copy of GDAL/OGR license:
  Copyright (c) 2000, Frank Warmerdam

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE. 
---------------------------------------------------------------------------*/
using System;
using System.Text;
using System.Runtime.InteropServices;
using System.Collections.Specialized;
using System.Configuration;

namespace Gdal
{

    internal enum CPLErr
    {
        None = 0,
        Debug = 1,
        Warning = 2,
        Failure = 3,
        Fatal = 4
    }

    public class GdalApi
    {
        public static bool GdalPresent
        {
            get
            {
                return InterOp.GdalPresent;
            }
        }
    }


    /// <summary>
    /// Common location for all GDAL InterOp method declarations
    /// </summary>
    internal class InterOp
    {

        private static bool mbGdalInstalled = false;
        public static bool GdalPresent
        {
            get
            {
                return mbGdalInstalled;
            }
        }

        static InterOp()
        {
            try
            {
                GDALAllRegister();
                mbGdalInstalled = true;
            }
            catch
            { 
                mbGdalInstalled = false; 
            }

        }

        const string gdalDll = "gdal";
        //const string gdalDll = "gdal.dll";

        [DllImport(gdalDll)]
        internal static extern IntPtr GDALOpen([MarshalAs(UnmanagedType.LPStr)] string pszFilename, Access eAccess);

        [DllImport(gdalDll)]
        internal static extern IntPtr GDALGetDatasetDriver(HandleRef hDataset);


        [DllImport(gdalDll)]
        internal static extern int GDALGetRasterXSize(HandleRef hDataset);

        [DllImport(gdalDll)]
        internal static extern int GDALGetRasterYSize(HandleRef hDataset);

        [DllImport(gdalDll)]
        internal static extern int GDALGetRasterCount(HandleRef hDataset);

        [DllImport(gdalDll, CharSet = CharSet.Ansi, EntryPoint = "GDALGetProjectionRef")]
        private static extern IntPtr GDALGetProjectionRefExt(HandleRef hDataset);

        internal static string GDALGetProjectionRef(HandleRef hDataset)
        {
            return Marshal.PtrToStringAnsi(GDALGetProjectionRefExt(hDataset));
        }

        [DllImport(gdalDll)]
        internal static extern CPLErr GDALGetGeoTransform(HandleRef hDataset, [In, Out] double[] Transform);

        [DllImport(gdalDll)]
        internal static extern CPLErr GDALSetGeoTransform(HandleRef hDataset, [In, Out] double[] Transform);


        [DllImport(gdalDll)]
        internal static extern void GDALClose(HandleRef hDriver);

        [DllImport(gdalDll, EntryPoint = "CPLGetLastErrorMsg")]
        private static extern IntPtr CPLGetLastErrorMsgExt();

        internal static string CPLGetLastErrorMsg()
        {
            return Marshal.PtrToStringAnsi(CPLGetLastErrorMsgExt());
        }

        [DllImport(gdalDll, EntryPoint = "CPLSetConfigOption")]
        private static extern IntPtr CPLSetConfigOption([MarshalAs(UnmanagedType.LPStr)] string key, [MarshalAs(UnmanagedType.LPStr)] string value);


        [DllImport(gdalDll)]
        internal static extern void GDALAllRegister();

        [DllImport(gdalDll)]
        internal static extern IntPtr GDALCreateCopy(HandleRef hDriver, [MarshalAs(UnmanagedType.LPStr)] string filename,
            HandleRef hDataset, int bStrict, HandleRef options, HandleRef progressFn, HandleRef progressData);

        [DllImport(gdalDll)]
        internal static extern IntPtr GDALCreate(HandleRef hDriver, [MarshalAs(UnmanagedType.LPStr)] string filename,
            int width, int height, int nBands, DataType type, HandleRef options);

        [DllImport(gdalDll)]
        internal static extern IntPtr GDALGetDriverByName([MarshalAs(UnmanagedType.LPStr)]string pszName);


        [DllImport(gdalDll)]
        internal static extern int GDALGetDriverCount();

        [DllImport(gdalDll)]
        internal static extern IntPtr GDALGetDriver(int iDriver);

        [DllImport(gdalDll, CharSet = CharSet.Ansi, EntryPoint = "GDALGetDriverShortName")]
        private static extern IntPtr GDALGetDatasetDriverExt(HandleRef hDataset);

        internal static string GDALGetDriverShortName(HandleRef hDriver)
        {
            return Marshal.PtrToStringAnsi(GDALGetDatasetDriverExt(hDriver));
        }

        [DllImport(gdalDll, CharSet = CharSet.Ansi, EntryPoint = "GDALGetDriverLongName")]
        private static extern IntPtr GDALGetDriverLongNameExt(HandleRef hDriver);

        internal static string GDALGetDriverLongName(HandleRef hDriver)
        {
            return Marshal.PtrToStringAnsi(GDALGetDriverLongNameExt(hDriver));
        }

        [DllImport(gdalDll, CharSet = CharSet.Ansi, EntryPoint = "GDALGetDescription")]
        private static extern IntPtr GDALGetDescriptionExt(HandleRef hDriver);

        internal static string GDALGetDescription(HandleRef hDriver)
        {
            return Marshal.PtrToStringAnsi(GDALGetDescriptionExt(hDriver));
        }

        [DllImport(gdalDll)]
        internal static extern void GDALDestroyDriverManager();

        [DllImport(gdalDll)]
        internal static extern IntPtr GDALGetRasterBand(HandleRef hDataset, int index);

        [DllImport(gdalDll)]
        internal static extern CPLErr GDALRasterIO(HandleRef hBand, RWFlag flag, int xOff, int yOff,
            int xSize, int ySize, IntPtr data, int bufXSize, int bufYSize, DataType type, int pixelStride,
            int lineStride);

        [DllImport(gdalDll)]
        internal static extern int GDALGetRasterColorInterpretation(HandleRef hBand);

        [DllImport(gdalDll)]
        internal static extern int GDALGetAccess(HandleRef hDataset);

        [DllImport(gdalDll, EntryPoint = "GDALGetMetadata")]
        private static extern IntPtr GDALGetMetadataExt(HandleRef hDataset,
            [MarshalAs(UnmanagedType.LPStr)] string domain);

        internal unsafe static /*Dictionary<string, string>*/StringDictionary GDALGetMetadata(HandleRef hDataset, string domain)
        {
            /*Dictionary<string, string>*/
            StringDictionary data = new /*Dictionary<string, string>*/StringDictionary();
            void** stringList = (void**)GDALGetMetadataExt(hDataset, domain);
            if (new IntPtr(stringList) != IntPtr.Zero)
            {
                while (new IntPtr(*stringList) != IntPtr.Zero)
                {
                    string keyValue = Marshal.PtrToStringAnsi(new IntPtr(*stringList));
                    string[] elements = keyValue.Split("=".ToCharArray());
                    if (elements.Length < 2)
                        throw new ApplicationException("Parse error while parsing string list in GDALGetMetadata()");
                    data.Add(elements[0], elements[1]);
                    ++stringList;
                }
            }
            return data;
        }

        [DllImport(gdalDll, EntryPoint = "GDALGetMetadataItem")]
        private static extern IntPtr GDALGetMetadataItemExt(HandleRef hDataset,
            [MarshalAs(UnmanagedType.LPStr)] string key, [MarshalAs(UnmanagedType.LPStr)] string domain);

        internal static string GDALGetMetadataItem(HandleRef hDataset, string key, string domain)
        {
            return Marshal.PtrToStringAnsi(GDALGetMetadataItemExt(hDataset, key, domain));
        }

        [DllImport(gdalDll)]
        internal static extern void GDALSetMetadataItem(HandleRef hDataset,
            [MarshalAs(UnmanagedType.LPStr)] string key, [MarshalAs(UnmanagedType.LPStr)] string value,
            [MarshalAs(UnmanagedType.LPStr)] string domain);
    }
}
